%% some basic start commands 
clc %clear the command window 
clear all %clear the workspace
close all %close all open figures
v = btkGetVersion % get the current BTK version 

%% Add the Joint Center of the right Knee to the Static Reference Trail as a virtual marker
%add a marker to the static reference trail
filename= ('Static_Example.c3d'); %the filename of the Static reference
acqstatic = btkReadAcquisition(filename); % read the static c3d file
[markersStatic, markersInfoStatic, markersResidualStatic] = btkGetMarkers(acqstatic) % get marker information from the static c3d file
%the output for example markersStatic is a maatlab structure with all
%labeled markers and its X Y and Z componets
%The knee joint center is often calculated as the midpoint between the
%medial and the lateral epicondylus marker
            x1=markersStatic.epi_med_right(:,1); % x componets medial epicondylus marker
            x2=markersStatic.epi_lat_right(:,1); % x componets lateral epicondylus marker
            y1=markersStatic.epi_med_right(:,2); % y componets medial epicondylus marker
            y2=markersStatic.epi_lat_right(:,2); % y componets lateral epicondylus marker
            z1=markersStatic.epi_med_right(:,3); % z componets medial epicondylus marker
            z2 =markersStatic.epi_lat_right(:,3);% z componets lateral epicondylus marker
            M_p = [(x1+x2)/2, (y1+y2)/2, (z1+z2)/2]; %calculate the midpoint half the way /2 of the two marker distance
            
[points, pointsInfo] = btkAppendPoint(acqstatic, 'marker' , 'V_Knee_Joint_Center_right',M_p, markersResidualStatic.epi_lat_right);
%append a point to the Acquisition with the name
%'V_Knee_Joint_Center_right' and the coordinates from the Matrix M_p,
%markersResidualStatic.epi_lat_right are the residuals of the new virtual marker and
%can simply be used from an existing marker
btkWriteAcquisition(acqstatic, 'Static_Example_with_Knee_Joint_Center_right.c3d') %write a new C3D file with the Virtual marker appended
%close the previously opened C3D file
btkCloseAcquisition(acqstatic)

%% Load a example file for Running and calculate Stance Time of the right leg based on the Force data
%% add Toutchdown and Toe off events to the C3D
%load a example file
filename= ('Running_Example.c3d'); %the filename of the running c3d
acqrunning = btkReadAcquisition(filename);  % read the running c3d file
%% get the marker data
[markers, markersInfo, markersResidual] = btkGetMarkers(acqrunning)
%markers contains all marker with its X Y and Z coordinates 
%% plot a marker trajectory
figure(1)
plot(markers.C_7) % plot the marker coordinates of the C_7 marker
legend ('X', 'Y', 'Z')
legend boxoff
box off
ylabel ('Coordinates in mm')
xlabel ('Frame Number Marker Data')
%% get the raw force data
[forceplates, forceplatesInfo] = btkGetForcePlatforms(acqrunning)
% get the Ground Reaction Forces
grw = btkGetGroundReactionWrenches(acqrunning)
%% plot the vertical Ground Reaction Force
figure(2)
plot(grw(2).F(:,end)) % in this example the participant hits the force plate 2 with the right leg
box off
xlabel ('Frame Number Force Data')
ylabel ('Vertical Ground Reaction Force')
%% calculate stance time based on ground reaction force
vGRF = grw(2).F(:,end);
%use a force threshold to determine the stance time
thresholdforce = 20% N when 20 N of the vGRF
[stance_frames_Analog_based]=find (vGRF>= thresholdforce) %stance_frames containg all Frames of the vGRF which are above 20N
stance_number_of_frames_Analog_based = length (stance_frames_Analog_based) %the total number of frames where the stance phase has be determined
analog_frequency = btkGetAnalogFrequency(acqrunning) % get the analof frequency in Hz of the Analogchannels
stance_time_Analog_based = stance_number_of_frames_Analog_based/analog_frequency % stance time in seconds

%% add Touchdown and Toe off as events into your C3d Files
%this must be based on the point frequency (the frequency of the markers)
btkClearEvents(acqrunning) %clear events if exist
%fet the first frame of the aqquistion
first_frame= btkGetFirstFrame(acqrunning)

af = btkGetAnalogFrequency(acqrunning); %analog frequency
pf = btkGetPointFrequency(acqrunning);  %marker frequency
fkratio = af/pf;
Down_sampled_Force = downsample(grw(2).F(:,end),fkratio); %The analoag data downsampled to the frequency of the markers

[stance_frames]=find (Down_sampled_Force >= thresholdforce) %stance_frames containg all Frames of the vGRF which are above 20N
stance_number_of_frames = length (stance_frames) %the total number of frames where the stance phase has be determined
analog_frequency = btkGetAnalogFrequency(acqrunning) % get the analof frequency in Hz of the Analogchannels
stance_time = stance_number_of_frames/pf % stance time in seconds
TD = stance_frames(1,1)-2;
TO = stance_frames(end,1);

dt = 1 / pf;
ff = btkGetFirstFrame(acqrunning);
td = ((ff+TD)*dt);%in c3d time format
to = ((ff+TO)*dt);%in c3d time format

btkAppendEvent(acqrunning, 'TD', td, 'Right');
btkAppendEvent(acqrunning, 'TO', to, 'Right');
btkWriteAcquisition(acqrunning, 'Running_Example_with_Events.c3d') % Please check the new written C3D file inthe folder showing events in the timebar
%% close the previously opened C3D file
btkCloseAcquisition(acqrunning)

%% have fun :)
S(1) = load('gong');
sound(S(1).y,S(1).Fs)
